/**
  ******************************************************************************
  * @file    External LED
  * @author  Lee Dyche - Tecsploit.com
  * @version V1.0.0
  * @date    25/04/2014
  * @brief   Controls tecsploits custom raspberry pi shaped led matrix, initialises GPIO pind D1 - D6
  * And toggles them high or low to control the LED's, runs through a few sample animations on the LED's
  ******************************************************************************
  * @attention
  * This is program is provided as is with no warranty!!
  ******************************************************************************
  */


/* Includes ------------------------------------------------------------------*/
#include "stm32f4_discovery.h"


/* Private typedef -----------------------------------------------------------*/
GPIO_InitTypeDef  GPIO_InitStructure;

/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void Delay(__IO uint32_t nCount);
void Flash(int count, int speed);
void Load(int step);
void BlowTop(int speed);
void ClearTop();
void ClearAll();
void LoadFromTop(int speed);
void Volcano(long step);

/* Private functions ---------------------------------------------------------*/




int main(void)
{
  /* GPIOD Periph clock enable */
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOD, ENABLE);
  //setup pins D1 - D6
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_1 | GPIO_Pin_2| GPIO_Pin_3| GPIO_Pin_4 | GPIO_Pin_5 | GPIO_Pin_6;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
  GPIO_Init(GPIOD, &GPIO_InitStructure);

  while (1)
  {
	  Volcano(9000000);
  }
}

/**
 * Sample method that runs a basic animation on the LED display, starting with a volcano and ending with some flashing
 * Uses several helper methods
 */
void Volcano(long step){
	  while (1)
	  {

		//turn on rows bottom to top
	    GPIO_SetBits(GPIOD, GPIO_Pin_1);
	    Delay(step);
	    GPIO_SetBits(GPIOD, GPIO_Pin_2);
	    Delay(step);
	    GPIO_SetBits(GPIOD, GPIO_Pin_3);
	    Delay(step);
	    GPIO_SetBits(GPIOD, GPIO_Pin_4);
	    Delay(step);

	    //animate the green leds in a loop 5 times
	    int i=0;
	    for(i=0; i<5; i++){
			BlowTop(step/5);
			Delay(step);
			ClearTop();
			Delay(step);
	    }

	    //turn the led rows off moving from top to bottom
	    GPIO_ResetBits(GPIOD, GPIO_Pin_4 );
		Delay(step);
	    GPIO_ResetBits(GPIOD, GPIO_Pin_3);
		Delay(step);
	    GPIO_ResetBits(GPIOD, GPIO_Pin_2);
		Delay(step);
	    GPIO_ResetBits(GPIOD, GPIO_Pin_1 );
		Delay(step);
		//turn leds on from top to bottom
		LoadFromTop(step);
		Delay(step);
		ClearAll();
		Delay(step);
		//flash leds 10 times very fast.
		Flash(10, step / 8);
	  }

}

void LoadFromTop(int speed){
    GPIO_SetBits(GPIOD, GPIO_Pin_6);
    Delay(speed);
    GPIO_SetBits(GPIOD, GPIO_Pin_5);
    Delay(speed);
    GPIO_SetBits(GPIOD, GPIO_Pin_4);
    Delay(speed);
    GPIO_SetBits(GPIOD, GPIO_Pin_3);
    Delay(speed);
    GPIO_SetBits(GPIOD, GPIO_Pin_2);
    Delay(speed);
    GPIO_SetBits(GPIOD, GPIO_Pin_1);
}

void BlowTop(int speed){
	 	 	GPIO_SetBits(GPIOD, GPIO_Pin_5);
		    Delay(speed);
		    GPIO_SetBits(GPIOD, GPIO_Pin_6);
}

void ClearTop(){
    GPIO_ResetBits(GPIOD, GPIO_Pin_5 | GPIO_Pin_6 );
}

void ClearAll(){
	GPIO_ResetBits(GPIOD, GPIO_Pin_1 | GPIO_Pin_2| GPIO_Pin_3| GPIO_Pin_4 | GPIO_Pin_5 | GPIO_Pin_6 );
}

void Load(int step){
	  while (1)
	  {

	    GPIO_SetBits(GPIOD, GPIO_Pin_1);
	    Delay(step);
	    GPIO_SetBits(GPIOD, GPIO_Pin_2);
	    Delay(step);
	    GPIO_SetBits(GPIOD, GPIO_Pin_3);
	    Delay(step);
	    GPIO_SetBits(GPIOD, GPIO_Pin_4);
	    Delay(step);
	    GPIO_SetBits(GPIOD, GPIO_Pin_5);
	    Delay(step);
	    GPIO_SetBits(GPIOD, GPIO_Pin_6);
	    Delay(step);

	    GPIO_ResetBits(GPIOD, GPIO_Pin_1 | GPIO_Pin_2| GPIO_Pin_3| GPIO_Pin_4 | GPIO_Pin_5 | GPIO_Pin_6 );

	    Delay(step);
	  }

}

void Flash(int count, int speed){
	  int i=0;
	  for(i = 0; i < count; i++)
	  {
	    GPIO_SetBits(GPIOD, GPIO_Pin_1 | GPIO_Pin_2| GPIO_Pin_3| GPIO_Pin_4 | GPIO_Pin_5 | GPIO_Pin_6);

	    Delay(speed);

	    GPIO_ResetBits(GPIOD, GPIO_Pin_1 | GPIO_Pin_2| GPIO_Pin_3| GPIO_Pin_4 | GPIO_Pin_5 | GPIO_Pin_6 );

	    Delay(speed);
	  }

}

/**
  * @brief  Delay Function.
  * @param  nCount:specifies the Delay time length.
  * @retval None
  */
void Delay(__IO uint32_t nCount)
{
  while(nCount--)
  {
  }
}

#ifdef  USE_FULL_ASSERT

/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t* file, uint32_t line)
{
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

  /* Infinite loop */
  while (1)
  {
  }
}
#endif
